import unittest
from bson import ObjectId
from pymongo import Connection


class UpdateOrderTest(unittest.TestCase):

    def setUp(self):

        c = Connection()
        c.test_database

        self.db = c.test_database

        self.data = {'_id': ObjectId('4f58782cbb69330d0a000000'),
                     '_types': ['Part'],
                     'attributes': [
                             {'display_name': 'foo1',
                              'value': 'foo1'},
                             {'display_name': 'foo2',
                              'value': 'foo2'}]}

        self.data_simple = {'_id': ObjectId('4f58782cbb69330d0a000000'),
                     '_types': 'Part',
                     'attributes': [
                             {'display_name': 'foo1',
                              'value': 'foo1'},
                             {'display_name': 'foo2',
                              'value': 'foo2'}]}

        self.update = {"$set": {"attributes.$.value": "bar"}}
        self.db.test.drop()


    def check_results(self):
        """Check the update worked as expected"""
        updated = self.db.test.find_one()

        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 1)
        self.assertEquals(updated["attributes"][0]['value'], "foo1")
        self.assertEquals(updated["attributes"][1]['value'], "bar")

    def test_query_order_1(self):
        self._id = self.db.test.save(self.data)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"_id": self._id, "_types": "Part", "attributes.display_name": "foo2"}
        self.db.test.update(query, self.update)

        self.check_results()

    def test_query_order_2(self):
        self._id = self.db.test.save(self.data)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"_id": self._id, "attributes.display_name": "foo2", "_types": "Part"}
        self.db.test.update(query, self.update)

        self.check_results()

    def test_query_order_3(self):
        self._id = self.db.test.save(self.data)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"attributes.display_name": "foo2", "_types": "Part", "_id": self._id}
        self.db.test.update(query, self.update)

        self.check_results()

    def test_query_order_simple_1(self):
        self._id = self.db.test.save(self.data_simple)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"_id": self._id, "_types": "Part", "attributes.display_name": "foo2"}
        self.db.test.update(query, self.update)

        self.check_results()

    def test_query_order_simple_2(self):
        self._id = self.db.test.save(self.data_simple)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"_id": self._id, "attributes.display_name": "foo2", "_types": "Part"}
        self.db.test.update(query, self.update)

        self.check_results()

    def test_query_order_simple_3(self):
        self._id = self.db.test.save(self.data_simple)
        self.assertEquals(self.db.test.find({"attributes.value": "bar"}).count(), 0)

        query = {"attributes.display_name": "foo2", "_types": "Part", "_id": self._id}
        self.db.test.update(query, self.update)

        self.check_results()



if __name__ == '__main__':
    unittest.main()
